@@ -9,7 +9,7 @@ BEGIN {
 	use Exporter ();
 	our @ISA       = qw(Exporter);
 	our @EXPORT    = ();
-	our $VERSION   = '0.02';
+	our $VERSION   = '0.03';
 	    $VERSION   = eval $VERSION;
 	my ($pkg_path) = __FILE__ =~ /(.*)\.pm/;
 
@@ -74,29 +74,17 @@ class Pipeline {
 		this.bidirectionalCorefs = bidirectionalCorefs;
 	}
 
+	public Pipeline(boolean silent, boolean bidirectionalCorefs, long startingID) {
+		this(silent, bidirectionalCorefs);
+
+		PipelineItem.initializeCounters(startingID, Long.MIN_VALUE);
+	}
+
 	public String getAnnotatorLog() throws IOException {
 		return readStream(stdoutStream) + readStream(stderrStream);
 	}
 
-	private int getIndexFromWTI(String wti) {
-		String[] halves = wti.split(":");
-		int           n = -1;
-
-		if(halves.length >= 2) {
-			try {
-				n = Integer.parseInt(halves[1].replaceAll("[^0-9]", "")) - 1;
-			} catch(Exception e) {
-				System.err.println("getIndexFromWTI: " + e);
-			}
-		}
-
-		if(n < 0)
-			System.err.println("getIndexFromWTI: " + wti + " not valid WORD_TAG_INDEX_FORMAT");
-
-		return n;
-	}
-
-	private void initPipeline() {
+	public void initPipeline() {
 		Properties props = new Properties();
 		props.put("annotators", "tokenize, ssplit, pos, parse, lemma, ner, dcoref");
 		pipeline = new StanfordCoreNLP(props, false);
@@ -153,12 +141,12 @@ class Pipeline {
 
 			if(dependencies != null) {
 				for(SemanticGraphEdge edge: dependencies.edgeList()) {
-					String govStr = edge.getGovernor().toString(IndexedWord.WORD_TAG_INDEX_FORMAT);
-					String depStr = edge.getDependent().toString(IndexedWord.WORD_TAG_INDEX_FORMAT);
+					//String govStr = edge.getGovernor().toString(IndexedWord.WORD_TAG_INDEX_FORMAT);
+					//String depStr = edge.getDependent().toString(IndexedWord.WORD_TAG_INDEX_FORMAT);
 					GrammaticalRelation rel = edge.getRelation();
 
-					int govTokenIndex = getIndexFromWTI(govStr);
-					int depTokenIndex = getIndexFromWTI(depStr);
+					int govTokenIndex = edge.getGovernor().index()  - 1; //getIndexFromWTI(govStr);
+					int depTokenIndex = edge.getDependent().index() - 1; //getIndexFromWTI(depStr);
 
 					if(govTokenIndex >= 0 && depTokenIndex >= 0 &&
 					   govTokenIndex < ptl.size() && depTokenIndex < ptl.size()
@@ -170,6 +158,8 @@ class Pipeline {
 							depTokenIndex,
 							rel
 						));
+					} else {
+						System.err.println("Index of " + edge.toString() + " out of range!");
 					}
 				}
 			}
@@ -228,10 +218,41 @@ class Pipeline {
 abstract class PipelineItem {
 	protected UUID   id    = null;
 	protected String idStr = "";
+	protected static long idLeast = Long.MIN_VALUE;
+	protected static long idMost  = Long.MIN_VALUE;
+
+	public void setIDFromString(String str) {
+		id = UUID.nameUUIDFromBytes(str.getBytes());
+	}
+
+	public static void initializeCounters(long least, long most) {
+		idLeast = least;
+		idMost  = most;
+	}
+
+	public static void randomizeCounters() {
+		UUID t = UUID.randomUUID();
+
+		PipelineItem.initializeCounters(
+			t.getLeastSignificantBits(),
+			t.getMostSignificantBits()
+		);
+	}
+
+	public static UUID generateID() {
+		if(idLeast <= Long.MAX_VALUE)
+			idLeast++;
+		else if(idMost <= Long.MAX_VALUE)
+			idMost++;
+		else // if this happens, you've got big problems
+			System.out.println("PipelineItem.generateID(): ran out of IDs!");
+
+		return new UUID(idMost, idLeast);
+	}
 
 	public UUID getID() {
 		if(id == null)
-			id = UUID.randomUUID();
+			id = PipelineItem.generateID();
 		return id;
 	}
 
@@ -274,6 +295,7 @@ class PipelineCoreference extends PipelineItem {
 		this.toHead       = toHead;
 		this.fromToken    = fromToken;
 		this.toToken      = toToken;
+		//this.setIDFromString(fromToken.getIDString() + toToken.getIDString());
 	}
 
 	public boolean equals(PipelineCoreference b) {
@@ -329,7 +351,7 @@ class PipelineDependency extends PipelineItem {
 		this.dependent    = dependent;
 		this.govIndex     = govIndex;
 		this.depIndex     = depIndex;
-		this.relation     = relation.getShortName();
+		this.relation     = relation.toString();
 		this.longRelation = relation.getLongName();
 	}
 
@@ -485,8 +507,8 @@ Lingua::StanfordCoreNLP - A Perl interface to Stanford's CoreNLP tool set.
  # Note that Lingua::StanfordCoreNLP can't be instantiated.
  use Lingua::StanfordCoreNLP;
 
- # Create a new NLP pipeline (silence messages, make corefs bidirectional)
- my $pipeline = new Lingua::StanfordCoreNLP::Pipeline(1, 1);
+ # Create a new NLP pipeline (don't silence messages, do make corefs bidirectional)
+ my $pipeline = new Lingua::StanfordCoreNLP::Pipeline(0, 1);
 
  # Process text
  # (Will output lots of debug info from the Java classes to STDERR.)
